;; ============================================================================
;; file:	test_midi_out.asm         (Philips Music Module MIDI Framework)
;; version:     0.7a (March 1, 2008)
;;
;; description: This is an example usage of the msx_midi Music Module MIDI
;;    	        framework. This example will route note on and note off
;;		messages received on midi channel 1 to midi channel 16.
;;
;;  This program is free software; you can redistribute it and/or modify
;;  it under the terms of the Lesser GNU General Public License as published by
;;  the Free Software Foundation; either version 2, or (at your option)
;;  any later version. See COPYING for more details.
;;
;; (C)2007 by Vincent van Dam (vincentd@erg.verweg.com)
;; ============================================================================

	db   $fe
	dw   bin_begin
	dw   bin_end
	dw   bin_start

	org  $9000

bin_begin:
	;; include the midi framework
	include "msx_midi.asm"

bin_start:
	;; --------------------------------------------------------------------
	;; initialise the framework
	;; --------------------------------------------------------------------

	xor   a		; use internal clock (vblank) as midi clock
	call  msx_midi_init

	;; add a handler for all note on events received on midi channel 1
	ld    hl,noteon
	ld    e,1	; midi channel (0 to responds on all channels)
	call  msx_midi_add_noteon_handler

	;; add a handler for all note off events received on midi channel 1
	ld    hl,noteoff
	ld    e,1	; midi channel (0 to responds on all channels)
	call  msx_midi_add_noteoff_handler

	;; --------------------------------------------------------------------
	;; main loop, wait for space key to exit program.
	;; --------------------------------------------------------------------
_wait:
	ei	; make sure interrupts are enabled to receive midi data
	
	;; msx_midi_loop needs to be included in your main application loop and
	;; needs to be called as much as possible. it handles the internal
	;; clock and handles the dispatching of midi messages.
	call  msx_midi_loop

	;; wait for space key
	in    a,($aa)
	and   240
	or    8
	out   ($aa),a
	in    a,($a9)
	and   1
	jr    nz,_wait

	;; --------------------------------------------------------------------
	;; deinitialize midi framework (restore interrupt handlers)
	;; --------------------------------------------------------------------

	call  msx_midi_exit
	ret
	
	;; ====================================================================
	;; handler methods. these methods are called whenever a specific midi
	;; message was received on the midi channel the handler was configured
	;; to listen to. register [a] will always contain the midi channel.
	;; register [b] and register [c] will always contain extra information.
	;; the information differs per message, see msx_midi.asm for more info.
	;; ====================================================================
	
	;; note on handler
	
noteon:
	ld    hl,_noteon_buffer+3
	ld    (hl),c
	ld    hl,_noteon_buffer
	call  msx_midi_midi_out
	ret
_noteon_buffer:
	db    3,0,$90,0,$7f

	;; note off handler
	
noteoff:
	ld    hl,_noteoff_buffer+3
	ld    (hl),c
	ld    hl,_noteoff_buffer
	call  msx_midi_midi_out
	ret
_noteoff_buffer:
	db    3,0,$80,0,$7f
	
bin_end:	
